acia_baud	equ	$E645
acia_control	equ	$E646
acia_data	equ	$E647

crtaddr 	equ	$E600
crtdata 	equ	$E601

synchro_addr	equ	$ED04
PUTCHAR		equ	$FA16

COPY		equ	1
DRAW_CHAR	equ	2
DRAW_LINE	equ	3
CLEAR_SCREEN	equ	4
WINDOW_SIZE	equ	5

SCRWIDTH	equ	40
SCRHEIGHT	equ	25

	org	$100
	
	jmp	start

sio_read
	ldab	#%00000001
sio_r1	bitb	acia_control
	beq	sio_r1
	ldaa	acia_data
	rts
	
sio_write
	ldab	#%00000010
sio_w1	bitb	acia_control
	bne	sio_w1
	staa	acia_data
	rts

initSio
	ldab	#%00000011	; init SIO
	stab	acia_control	;

	ldab	#%00000111	; set baud 19200
	stab	acia_baud	; not SIO, specific Pyldin reg
	ldab	#%01010001	; 8 bit, 2 stop, 1:16f
	stab	acia_control	; set SIO
	rts

start
	ldx	#logo1
	int	$23

	int	$20

	jsr	initSio

mainLoop
	jsr	sio_read
	sei
	cmpa	#COPY
	bne	main1
	jsr	ccopy
	jmp	mainNext
main1	cmpa	#DRAW_CHAR
	bne	main2
	jsr	cdrawchar
	jmp	mainNext
main2	cmpa	#DRAW_LINE
	bne	main3
	jsr	cdrawline
	jmp	mainNext
main3	cmpa	#CLEAR_SCREEN
	bne	main4
	jsr	cclearscreen
	jmp	mainNext
main4	cmpa	#WINDOW_SIZE
	bne	main5
	jsr	cwindowsize
	jmp	mainNext
main5
mainNext
	ldaa	#$0
	jsr	sio_write
	jmp	mainLoop
	
tcopy	db	10, 'COPY',0
ccopy
	jsr	sio_read
	staa	cx
	jsr	sio_read
	staa	cy
	jsr	sio_read
	staa	cx1
	jsr	sio_read
	staa	cy1
	jsr	sio_read
	staa	nx
	jsr	sio_read
	staa	ny
	cli
	jmp	fastCP
	rts
	
tdrawchar db	10, 'DRAW_CHAR',0
cdrawchar
	jsr	sio_read
	staa	cx
	jsr	sio_read
	staa	cy
	jsr	sio_read
	staa	char
	cli
	ldaa	cx
	ldab	cy
	int	$15
	ldaa	char
	int	$22
	rts
	
tdrawline db	10, 'DRAW_LINE',0
cdrawline
	jsr	sio_read
	staa	cx
	jsr	sio_read
	staa	cy
	jsr	sio_read
	staa	nx
	jsr	sio_read
	staa	nx+1
	ldx	#buffer
	stx	ny
	ldx	nx
cdrawl1	stx	nx
	ldx	ny
	jsr	sio_read
	staa	0,x
	inx
	stx	ny
	ldx	nx
	dex
	bne	cdrawl1
	ldx	ny
	clr	0,x
	cli
	ldaa	cx
	ldab	cy
	int	$15
	ldx	#buffer
	jmp	fastDL
	int	$23
	rts
	
tcls	db	$0c
cclearscreen
	cli
	ldx	#tcls
	int	$23
	rts
	
twinsize db	10, 'WINDOW_SIZE',0
cwindowsize
	rts
	
logo1
	db	$0c,$1e,25,25,'Welcome to SIO PC terminal!'
	db	$1e,30,30,'Press any key!',0

	
fastDL
	asl	cy
	ldaa	#scanlin
	ldab	#scanlin/256
	adda	cy
	adcb	#0
	staa	x1+1
	stab	x1
	ldx	x1
	ldx	0,x
	stx	x1
	ldaa	x1+1
	ldab	x1
	adda	cx
	adcb	#0
	staa	x1+1
	stab	x1
	andb	#$0F
	orab	#$20
	staa	x2+1
	stab	x2
	ldab	cx
	sei
	sts	sp
	lds	#buffer-1
fastDL1	pula
	tsta
	beq	fastDL2
	cmpb	#SCRWIDTH
	bcc	fastDL3
	ldx	x1
	staa	0,x
	inx
	stx	x1
	ldx	x2
	staa	0,x
	inx
	stx	x2
fastDL3	incb
	bra	fastDL1
fastDL2	lds	sp
	cli
	rts

fastCP
	ldaa	cx1
	cmpa	#SCRWIDTH
	bcs	fastCP5
	ldaa	#SCRWIDTH-1
fastCP5	suba	cx
	inca
	staa	cx1
	
	ldaa	cy1
	suba	cy
	inca
	staa	cy1
	
	asl	cy
	ldaa	#scanlin
	ldab	#scanlin/256
	adda	cy
	adcb	#0
	staa	x1+1
	stab	x1
	ldx	x1
	ldx	0,x
	stx	x1
	ldaa	x1+1
	ldab	x1
	adda	cx
	adcb	#0
	staa	x1+1
	stab	x1
	andb	#$0F
	orab	#$20
	staa	x2+1
	stab	x2

	asl	ny
	ldaa	#scanlin
	ldab	#scanlin/256
	adda	ny
	adcb	#0
	staa	x3+1
	stab	x3
	ldx	x3
	ldx	0,x
	stx	x3
	ldaa	x3+1
	ldab	x3
	adda	nx
	adcb	#0
	staa	x3+1
	stab	x3
	andb	#$0F
	orab	#$20
	staa	x4+1
	stab	x4
	sei
	sts	sp

fastCP0	ldab	cx1
	ldx	x3
	stx	dsts
	ldx	x4
	stx	dstb
	lds	x2
	des
fastCP1	pula
	ldx	dsts
	staa	0,x
	inx
	stx	dsts
	ldx	dstb
	staa	0,x
	inx
	stx	dstb
	decb
	bne	fastCP1
	ldaa	x2+1
	ldab	x2
	adda	#SCRWIDTH+2
	adcb	#0
	staa	x2+1
	stab	x2
	ldaa	x3+1
	ldab	x3
	adda	#SCRWIDTH+2
	adcb	#0
	staa	x3+1
	stab	x3
	andb	#$0F
	orab	#$20
	staa	x4+1
	stab	x4
	dec	cy1
	bne	fastCP0
	lds	sp
	cli
	rts
		
scanlin	dw	$f000, $f02a, $f054, $f07e, $f0a8, $f0d2, $f0fc, $f126, $f150, $f17a
	dw	$f1a4, $f1ce, $f1f8, $f222, $f24c, $f276, $f2a0, $f2ca, $f2f4, $f31e
        dw	$f348, $f372, $f39c, $f3c6, $f3f0, $f41a, $f444, $f46e, $f498, $f4c2
        dw	$f4ec, $f516, $f540, $f56a, $f594, $f5be, $f5e8, $f612, $f63c, $f666
        dw	$f690, $f6ba, $f6e4, $f70e, $f738, $f762, $f78c, $f7b6, $f7e0, $f80a
        dw	$f834, $f85e, $f888, $f8b2, $f8dc, $f906, $f930, $f95a, $f984, $f9ae
        dw	$f9d8, $fa02, $fa2c, $fa56, $fa80, $faaa, $fad4, $fafe, $fb28, $fb52
        dw	$fb7c, $fba6, $fbd0, $fbfa, $fc24, $fc4e, $fc78, $fca2, $fccc, $fcf6

	
cx	dw	0
cy	dw	0
cx1	dw	0
cy1	dw	0
nx	dw	0
ny	dw	0
char	db	0

x1	dw	0
x2	dw	0
x3	dw	0
x4	dw	0
sp	dw	0
curX	dw	0
dsts	dw	0
dstb	dw	0

buffer	ds 128,0
offscr
